package com.wizarpos.printermodeldemo;

import android.app.Activity;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.view.View;
import android.widget.TextView;

import com.cloudpos.jniinterface.PrinterInterface;
import com.wizarpos.htmllibrary.PrintHtmlInterface;
import com.wizarpos.htmllibrary.PrinterBitmapUtil;
import com.wizarpos.htmllibrary.PrinterHtmlListener;
import com.wizarpos.htmllibrary.PrinterHtmlUtil;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.util.Map;

public class MainActivity extends AppCompatActivity {
    private TextView textView;
    private boolean isOpened;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        textView = findViewById(R.id.textview);
        textView = (TextView) findViewById(R.id.textview);
        textView.setMovementMethod(ScrollingMovementMethod.getInstance());
    }


    public void open(View view) {
        if (isOpened) {
            sendFailedMsg(getResources().getString(R.string.device_opened));
        } else {
            try {
                int result = PrinterInterface.open();
                if (result < 0) {
                    sendFailedMsg(getResources().getString(R.string.operation_with_error) + result);
                } else {
                    isOpened = true;
                    sendSuccessMsg(getResources().getString(R.string.operation_successful));
                }
            } catch (Throwable e) {
                e.printStackTrace();
                sendFailedMsg(getResources().getString(R.string.operation_failed));
            }
        }
    }

    public void close(View view) {
        if (isOpened) {
            isOpened = false;
            int result = PrinterInterface.close();
            checkResult(result);
            sendSuccessMsg(getResources().getString(R.string.operation_successful));
        }
    }

    public void queryStatus(View view) {
        if (isOpened) {
            int result = PrinterInterface.queryStatus();
            if (result > 0) {
                // has paper
                sendSuccessMsg("PAPER_ON");
            } else if (result == 0) {
                // no paper
                sendFailedMsg("PAPER_OUT");
            }
            checkResult(result);
        }
    }


    public void writeData(View view) {
        if (isOpened) {
            Bitmap bitmap =null;// = BitmapFactory.decodeResource(getResources(), R.drawable.cloudpos);
            try {
                bitmap =  BitmapFactory.decodeStream(  getAssets().open("cloudpos.png"));
            } catch (IOException e) {
                e.printStackTrace();
            }

            byte[] arryBeginText = null;
            byte[] arryEndText = null;
            try {
                arryBeginText = getResources().getString(R.string.print_QR_code).getBytes("GB2312");
                arryEndText = "arryBeginText next This is a Bitmap of Barcode arryEndText".getBytes("GB2312");
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
                sendFailedMsg(getResources().getString(R.string.operation_failed));
            }
            begin();
            write(arryBeginText);
            // print line break
            writeLineBreak(2);
            PrinterBitmapUtil.printBitmap(bitmap, 0, 0, true);
            writeLineBreak(2);
            write(arryEndText);
            writeLineBreak(2);
            end();
        }
    }

    public void printHTML(View view) {
//        Activity activity = (Activity) this;
//        activity.runOnUiThread(new Runnable() {
//            @Override
//            public void run() {
//
//            }
//        });
        if (isOpened) {

            try {
                sendSuccessMsg(getResources().getString(R.string.printing_wait));
                InputStream inStream = getAssets().open("template11.html");
                String htmlContent = inputStream2String(inStream);
                PrintHtmlInterface printHtmlInterface = new PrinterHtmlUtil();
                printHtmlInterface.printHTML(getApplicationContext(), htmlContent, new PrinterHtmlListener() {
                    @Override
                    public void onGet(Bitmap bitmap, int errorCode) {

                    }

                    @Override
                    public void onFinishPrinting(int errorCode) {
                        if (errorCode == PRINT_SUCCESS) {
                            sendSuccessMsg(getResources().getString(R.string.printhtml_successful));
                        } else {
                            sendFailedMsg(getResources().getString(R.string.printhtml_failed));
                        }
                    }
                });
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    private void begin() {
        if (isOpened) {
            int result = PrinterInterface.begin();
            checkResult(result);
        }
    }

    private void end() {
        if (isOpened) {
            int result = PrinterInterface.end();
            checkResult(result);
        }
    }

    // print line break
    private void writeLineBreak(int lineNumber) {
        // write(new byte[]{'\n'});
        write(PrinterCommand.getCmdEscDN(lineNumber));
    }

    private void write(byte[] arryData) {
        int result = 0;
        if (arryData == null) {
            result = PrinterInterface.write(null, 0);
        } else {
            // byte [] testData = new byte[arryData.length + 10];
            // System.arraycopy(arryData, 0, testData, 10,
            // arryData.length);
            // result = PrinterInterface.write(testData, 10,
            // arryData.length);
            result = PrinterInterface.write(arryData, arryData.length);
            //checkResult(result);
        }

    }

    private void checkResult(int result) {
        if (result < 0) {
            sendFailedMsg(getResources().getString(R.string.operation_with_error) + result);
        } else {
            sendSuccessMsg(getResources().getString(R.string.operation_successful));
        }
    }

    //util
    private void textviewScorll(final String msg) {
        textView.post(new Runnable() {
            @Override
            public void run() {
                textView.append(msg);
                int scrollAmount = textView.getLayout().getLineTop(textView.getLineCount()) - textView.getHeight();
                if (scrollAmount > 0)
                    textView.scrollTo(0, scrollAmount);
                else
                    textView.scrollTo(0, 0);
            }
        });
    }

    private void sendFailedMsg(String msg) {
        textviewScorll(msg + "\n");
    }

    private void sendSuccessMsg(String msg) {
        textviewScorll(msg + "\n");
    }

    private String inputStream2String(InputStream inStream) {
        InputStreamReader reader = new InputStreamReader(inStream);
        StringBuilder sb = new StringBuilder();
        try {
            char[] cs = new char[1024];
            int len = -1;
            while ((len = reader.read(cs)) != -1) {
                sb.append(cs, 0, len);
            }
            reader.close();
        } catch (Exception e) {

        }
        return sb.toString();
    }
}
